@@ -16,8 +16,6 @@
  * Boston, MA 021110-1307, USA.
  */
 
-#define _XOPEN_SOURCE 500
-#define _GNU_SOURCE 1
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -30,7 +28,6 @@
 #include "print-tree.h"
 #include "transaction.h"
 #include "list.h"
-#include "version.h"
 #include "utils.h"
 
 #define FIELD_BUF_LEN 80
@@ -112,6 +109,7 @@ static void print_usage(void)
 		"to corrupt and a root+key for the item)\n");
 	fprintf(stderr, "\t-D Corrupt a dir item, must specify key and field\n");
 	fprintf(stderr, "\t-d Delete this item (must specify -K)\n");
+	fprintf(stderr, "\t-r Operate on this root (only works with -d)\n");
 	exit(1);
 }
 
@@ -160,7 +158,7 @@ static int corrupt_keys_in_block(struct btrfs_root *root, u64 bytenr)
 	struct extent_buffer *eb;
 
 	eb = read_tree_block(root, bytenr, root->leafsize, 0);
-	if (!eb)
+	if (!extent_buffer_uptodate(eb))
 		return -EIO;;
 
 	corrupt_keys(NULL, root, eb);
@@ -288,7 +286,7 @@ static void btrfs_corrupt_extent_tree(struct btrfs_trans_handle *trans,
 		next = read_tree_block(root, btrfs_node_blockptr(eb, i),
 				       root->leafsize,
 				       btrfs_node_ptr_generation(eb, i));
-		if (!next)
+		if (!extent_buffer_uptodate(next))
 			continue;
 		btrfs_corrupt_extent_tree(trans, root, next);
 		free_extent_buffer(next);
@@ -696,7 +694,7 @@ static int corrupt_metadata_block(struct btrfs_root *root, u64 block,
 	}
 
 	eb = read_tree_block(root, block, root->leafsize, 0);
-	if (!eb) {
+	if (!extent_buffer_uptodate(eb)) {
 		fprintf(stderr, "Couldn't read in tree block %s\n", field);
 		return -EINVAL;
 	}
@@ -845,27 +843,6 @@ out:
 	return ret;
 }
 
-static struct option long_options[] = {
-	/* { "byte-count", 1, NULL, 'b' }, */
-	{ "logical", 1, NULL, 'l' },
-	{ "copy", 1, NULL, 'c' },
-	{ "bytes", 1, NULL, 'b' },
-	{ "extent-record", 0, NULL, 'e' },
-	{ "extent-tree", 0, NULL, 'E' },
-	{ "keys", 0, NULL, 'k' },
-	{ "chunk-record", 0, NULL, 'u' },
-	{ "chunk-tree", 0, NULL, 'U' },
-	{ "inode", 1, NULL, 'i'},
-	{ "file-extent", 1, NULL, 'x'},
-	{ "metadata-block", 1, NULL, 'm'},
-	{ "field", 1, NULL, 'f'},
-	{ "key", 1, NULL, 'K'},
-	{ "item", 0, NULL, 'I'},
-	{ "dir-item", 0, NULL, 'D'},
-	{ "delete", 0, NULL, 'd'},
-	{ 0, 0, 0, 0}
-};
-
 /* corrupt item using NO cow.
  * Because chunk recover will recover based on whole partition scaning,
  * If using COW, chunk recover will use the old item to recover,
@@ -1018,7 +995,6 @@ int main(int ac, char **av)
 	/* chunk offset can be 0,so change to (u64)-1 */
 	u64 logical = (u64)-1;
 	int ret = 0;
-	int option_index = 0;
 	u64 copy = 0;
 	u64 bytes = 4096;
 	int extent_rec = 0;
@@ -1032,6 +1008,7 @@ int main(int ac, char **av)
 	u64 metadata_block = 0;
 	u64 inode = 0;
 	u64 file_extent = (u64)-1;
+	u64 root_objectid = 0;
 	char field[FIELD_BUF_LEN];
 
 	field[0] = '\0';
@@ -1040,8 +1017,31 @@ int main(int ac, char **av)
 
 	while(1) {
 		int c;
-		c = getopt_long(ac, av, "l:c:b:eEkuUi:f:x:m:K:IDd", long_options,
-				&option_index);
+		int option_index = 0;
+		static const struct option long_options[] = {
+			/* { "byte-count", 1, NULL, 'b' }, */
+			{ "logical", 1, NULL, 'l' },
+			{ "copy", 1, NULL, 'c' },
+			{ "bytes", 1, NULL, 'b' },
+			{ "extent-record", 0, NULL, 'e' },
+			{ "extent-tree", 0, NULL, 'E' },
+			{ "keys", 0, NULL, 'k' },
+			{ "chunk-record", 0, NULL, 'u' },
+			{ "chunk-tree", 0, NULL, 'U' },
+			{ "inode", 1, NULL, 'i'},
+			{ "file-extent", 1, NULL, 'x'},
+			{ "metadata-block", 1, NULL, 'm'},
+			{ "field", 1, NULL, 'f'},
+			{ "key", 1, NULL, 'K'},
+			{ "item", 0, NULL, 'I'},
+			{ "dir-item", 0, NULL, 'D'},
+			{ "delete", 0, NULL, 'd'},
+			{ "root", 0, NULL, 'r'},
+			{ NULL, 0, NULL, 0 }
+		};
+
+		c = getopt_long(ac, av, "l:c:b:eEkuUi:f:x:m:K:IDdr:",
+				long_options, &option_index);
 		if (c < 0)
 			break;
 		switch(c) {
@@ -1068,6 +1068,7 @@ int main(int ac, char **av)
 				break;
 			case 'U':
 				chunk_tree = 1;
+				break;
 			case 'i':
 				inode = arg_strtou64(optarg);
 				break;
@@ -1096,9 +1097,13 @@ int main(int ac, char **av)
 				break;
 			case 'I':
 				corrupt_item = 1;
+				break;
 			case 'd':
 				delete = 1;
 				break;
+			case 'r':
+				root_objectid = arg_strtou64(optarg);
+				break;
 			default:
 				print_usage();
 		}
@@ -1207,9 +1212,25 @@ int main(int ac, char **av)
 		ret = corrupt_btrfs_item(root, &key, field);
 	}
 	if (delete) {
+		struct btrfs_root *target = root;
+
 		if (!key.objectid)
 			print_usage();
-		ret = delete_item(root, &key);
+		if (root_objectid) {
+			struct btrfs_key root_key;
+
+			root_key.objectid = root_objectid;
+			root_key.type = BTRFS_ROOT_ITEM_KEY;
+			root_key.offset = (u64)-1;
+
+			target = btrfs_read_fs_root(root->fs_info, &root_key);
+			if (IS_ERR(target)) {
+				fprintf(stderr, "Couldn't find root %llu\n",
+					(unsigned long long)root_objectid);
+				print_usage();
+			}
+		}
+		ret = delete_item(target, &key);
 		goto out_close;
 	}
 	if (key.objectid || key.offset || key.type) {
